/***

cfconv.c	- conversion utility for adaptation of 1.1.0 configuration,
		  filter, and database files to 1.2.0
		  
Copyright (c) Gerard Paul Java 1998

This program should be run when upgrading from IPTraf 1.1.0 to 1.2.0.
IPTraf 1.2.0 contains a few new configurable items, and an additional field
to TCP and UDP filter parameters.  Not running this program will cause
IPTraf 1.2.0 to incorrectly interpret the earlier version's files and result
in incorrect information

***/

#include <stdio.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <fcntl.h>
#include <unistd.h>
#include <string.h>
#include "attrs.h"
#include "dirs.h"
#include "include-1.1.0/options.h"
#include "include-1.1.0/utfdefs.h"
#include "options.h"
#include "fltmgr.h"

int adaptconfig(void)
{
    struct OPTIONS110 options110;
    struct OPTIONS options120;
    char newname[60];
    
    int fd;
    
    printf("Converting configuration file\n");
    
    fd = open(CONFIGFILE, O_RDONLY);
    if (fd < 0) {
        fprintf(stderr, "Unable to open 1.1.0 configuration file\n");
        return 1;
    }
    
    if (read(fd, (char *) &options110, sizeof(struct OPTIONS110)) < 0) {
        fprintf(stderr, "Unable to read configuration file\n");
        close(fd);
        return 1;
    }
   
    close(fd);
    
    strcpy(newname, CONFIGFILE);
    rename(CONFIGFILE, strcat(newname, "-1.1.0"));
    
    options120.color = options110.color;
    options120.logging = options110.logging;
    options120.revlook = options110.revlook;
    options120.promisc = options110.promisc;
    options120.servnames = 0;
    options120.timeout = options110.timeout;
    options120.logspan = options110.logspan;

    fd = open(CONFIGFILE, O_WRONLY | O_TRUNC | O_CREAT, S_IRUSR | S_IWUSR);
    if (fd < 0) {
        fprintf(stderr, "Unable to create new configuration file\n");
        return 1;
    }
    
    if (write(fd, (char *) &options120, sizeof(struct OPTIONS)) < 0) {
        fprintf(stderr, "Unable to write new configuration record: defaults will be used\n");
        close(fd);
        return 1;
    }
    
    close(fd);
    return 0;
}
        
int convertfilterfile(struct ffnode *ftmp)
{
    int fd;
    int nfd;
    char filename[60];
    char backfile[60];
    int br = 0;
    struct hostparams110 hp110;
    struct hostparams hp120;
    
    strcpy(filename, WORKDIR);
    strcpy(backfile, WORKDIR);
    
    strcat(filename, ftmp->ffe.filename);
    strcpy(backfile, filename);
    strcat(backfile, ".110");
    
    printf("%s...", filename);
    
    rename(filename, backfile);
    
    fd = open(backfile, O_RDONLY);
    
    if (fd < 0) {
        fprintf(stderr, "Unable to open filter file %s\n", filename);
        rename(backfile, filename);
        return 1;
    }
    
    nfd = open(filename, O_WRONLY | O_TRUNC | O_CREAT, S_IRUSR | S_IWUSR);
    
    if (nfd < 0) {
        fprintf(stderr, "Unable to open file for modified filters in %s, skipping file\n", filename);
        close(fd);
        rename(backfile, filename);
        return 1;
    }
    
    do {
        br = read(fd, &hp110, sizeof(struct hostparams110));
        
        if (br > 0) {
            strcpy(hp120.s_fqdn, hp110.s_fqdn);
            strcpy(hp120.d_fqdn, hp110.d_fqdn);
            strcpy(hp120.s_mask, hp110.s_mask);
            strcpy(hp120.d_mask, hp110.d_mask);
            hp120.sport = hp110.sport;
            hp120.dport = hp110.dport;
            hp120.reverse = 'I';          /* 1.1.0 filter behavior */
        
            if (write(nfd, &hp120, sizeof(struct hostparams)) < 0) {
                fprintf(stderr, "Unable to write filter record for %s, skipping this file\n", filename);
                close(fd);
                close(nfd);
                unlink(filename);
                rename(backfile, filename);
                return 1;
            }
        } else if (br < 0) {
            fprintf(stderr, "Unable to read filter record from %s, skipping file\n", filename);
            close(fd);
            close(nfd);
	    unlink(filename);
            rename(backfile, filename);
            return 1;
        }
    } while (br > 0);
    
    close(fd);
    printf("\n");
    return 0;
}
            
    
int upgradefilters(struct ffnode *fltlist)
{
    struct ffnode *ftmp = fltlist;
    int r = 0;
    
    while (ftmp != NULL) {
        if (convertfilterfile(ftmp))
           r = 1;
        ftmp = ftmp->next_entry;
   }
   
   return r;
}    
    
        
int adaptfilters(void)
{
    struct ffnode *fltlist;
    int tr = 0, ur = 0;
    
    printf("Converting TCP filters\n");
    loadfilterlist(F_TCP, &fltlist);
    tr = upgradefilters(fltlist);
    
    destroyfilterlist(fltlist);

    printf("Converting UDP filters\n");    
    loadfilterlist(F_UDP, &fltlist);
    ur = upgradefilters(fltlist);
    
    destroyfilterlist(fltlist);
    
    return (tr || ur);
}    
        
int main(int argc, char **argv)
{
    int fd;
    char UPGRADETAG[40] = WORKDIR;
    
    printf("\nIPTraf Adaptaton Program 1.1.0-1.2.0\n\n");
    
    strcat(UPGRADETAG, "1.2.0.upgr");
    
    if ((fd = open(UPGRADETAG, O_RDONLY)) != -1) {
        fprintf(stderr, "IPTraf data files already converted\n");
        close(fd);
        return 1;
    }
    
    if (adaptconfig() || adaptfilters())
        fprintf(stderr, "Warning: upgrade incomplete; configuration and/or filters may malfunction\n");
    else {
        fd = creat(UPGRADETAG, S_IRUSR | S_IWUSR);
        close(fd);
    }
        
    return 0;
}
